-- Leg looks like:
--
-- body -|alfa|- coxa -(beta)- femur -(gama)- tibia
--

keys = {"leg/fr/", "leg/rr/", "leg/rl/", "leg/fl/"}

legs = {"leg/fr/alfa", "leg/fl/alfa", "leg/rr/alfa", "leg/rl/alfa"}

alfas = {"leg/fr/alfa", "leg/fl/alfa", "leg/rr/alfa", "leg/rl/alfa"}
betas = {"leg/fr/beta", "leg/fl/beta", "leg/rr/beta", "leg/rl/beta"}
gamas = {"leg/fr/gama", "leg/fl/gama", "leg/rr/gama", "leg/rl/gama"}

dirs = {{1,-1,0}, {1,1,0}, {-1,1,0}, {-1,-1,0}}

goal_angles = {0.785, 3.1415, 3.1415}

Chain:new(function()
	print("enforce 0 on alfas")
	joint_goto{joints=alfas, pos=0., time=1.}
	block_yield{joints=alfas}

	print("partly lift betas and move gamas to zero")
	joint_goto{joints=betas, pos=0.785398, time=3}
	block_yield{joints=betas}
	joint_spin{joints=gamas, velocity=0.2, time=1}
	block_yield{joints=gamas}

	joint_goto{joints=gamas, pos=0., time=2.}
	block_yield{joints=gamas}

	joint_goto{joints=alfas, pos=0.2, time=2.}
	block_yield{joints=alfas}

	print("straightn the legs")
	joint_goto{joints=betas, pos=1.5708, time=3}
	joint_goto{joints=gamas, pos=1.5708, time=3}
	block_yield{joints=betas}
	block_yield{joints=gamas}

	print("straightn the legs")
	joint_goto{joints=betas, pos=1.5708 + 0.78398, time=3}
	joint_goto{joints=gamas, pos=1.5708 + 1.5708, time=3}
	block_yield{joints=betas}
	block_yield{joints=gamas}
end):exec()

function sleek_body_move(vec)
	steps = 10
	for i=1,steps do
		body_move{dir={vec[1]/steps, vec[2]/steps, vec[3]/steps}, rot={{0,0,1}, 0}, time=0.5}
		block_yield{joints=alfas}
		block_yield{joints=betas}
		block_yield{joints=gamas}
	end
end

for i, key in ipairs(keys) do
	b_step = 0.05

	sleek_body_move({-dirs[i][1]*b_step, 0, 0})

	tip_move{leg = legs[i], dir={0,0,0.05}, time=2. }
	block_yield{joints={alfas[i], betas[i], gamas[i]}}

	state = get_leg_state{leg=legs[i]}
	pos = get_tip_pos{leg=legs[i], angles=state}
	desired_pos = get_tip_pos{leg=legs[i], angles=goal_angles}

	tip_goto{leg=legs[i], target={ desired_pos[1] + dirs[i][1]*b_step, desired_pos[2], pos[3] }, time=2.}
	block_yield{joints={alfas[i], betas[i], gamas[i]}}
	
	tip_move{leg = legs[i], dir={0,0,-0.05}, time=2. }
	block_yield{joints={alfas[i], betas[i], gamas[i]}}

	sleek_body_move({dirs[i][1]*b_step, 0, 0})
end
