#include "fw/base.h"

#include <schpin_lib_exp/protocol/config.h>

#pragma once

namespace schpin
{

enum servo_config_enum : uint8_t
{
        SC_INPUT_RANGE_LOW,
        SC_INPUT_RANGE_HIGH,
        SC_SERVO_RANGE_LOW,
        SC_SERVO_RANGE_HIGH,
        SC_ANGLE_CHANGE_REPORT,
        SC_ANGLE_STALL_RANGE,
        SC_GOTO_P_GAIN,
        SC_GOTO_I_GAIN,
        SC_GOTO_D_GAIN,
};

struct servo_config
{
        bangle      input_low;
        bangle      input_high;
        lewan_angle servo_low;
        lewan_angle servo_high;
        lewan_angle angle_change_report = lewan_angle::get< 5 >();
        lewan_angle angle_stall_range   = lewan_angle::get< 25 >();
        pid_gain    goto_p_gain         = pid_gain::from_float( 3.f );
        pid_gain    goto_i_gain         = pid_gain::from_float( 0.2f );
        pid_gain    goto_d_gain         = pid_gain::from_float( 0.02f );

        auto bind_prot_config()
        {
                return std::make_tuple(
                    prot_conf_parameter< SC_INPUT_RANGE_LOW >( &input_low ),
                    prot_conf_parameter< SC_INPUT_RANGE_HIGH >( &input_high ),
                    prot_conf_parameter< SC_SERVO_RANGE_LOW >( &servo_low ),
                    prot_conf_parameter< SC_SERVO_RANGE_HIGH >( &servo_high ),
                    prot_conf_parameter< SC_ANGLE_CHANGE_REPORT >( &angle_change_report ),
                    prot_conf_parameter< SC_ANGLE_STALL_RANGE >( &angle_stall_range ),
                    prot_conf_parameter< SC_GOTO_P_GAIN >( &goto_p_gain ),
                    prot_conf_parameter< SC_GOTO_I_GAIN >( &goto_i_gain ),
                    prot_conf_parameter< SC_GOTO_D_GAIN >( &goto_d_gain ) );
        }
};

using servo_master_config_message =
    typename prot_config_types< servo_config >::config_master_message;
using servo_master_config_variant =
    typename prot_config_types< servo_config >::config_master_command_variant;
using master_servo_config_message =
    typename prot_config_types< servo_config >::master_config_message;
using master_servo_config_variant =
    typename prot_config_types< servo_config >::master_config_command_variant;

}  // namespace schpin
