#include "fw/lewan/base.h"

#include <emlabcpp/static_circular_buffer.h>
#include <schpin_lib_exp/protocol.h>
#include <schpin_lib_exp/serialize.h>

#pragma once

namespace schpin
{
enum lewan_message_enum : uint8_t
{
        LEWAN_SERVO_MOVE_TIME_WRITE      = 1,
        LEWAN_SERVO_MOVE_TIME_READ       = 2,
        LEWAN_SERVO_MOVE_TIME_WAIT_WRITE = 7,
        LEWAN_SERVO_MOVE_TIME_WAIT_READ  = 8,
        LEWAN_SERVO_MOVE_START           = 11,
        LEWAN_SERVO_MOVE_STOP            = 12,
        LEWAN_SERVO_ID_WRITE             = 13,
        LEWAN_SERVO_ID_READ              = 14,
        LEWAN_SERVO_ANGLE_OFFSET_ADJUST  = 17,
        LEWAN_SERVO_ANGLE_OFFSET_WRITE   = 18,
        LEWAN_SERVO_ANGLE_OFFSET_READ    = 19,
        LEWAN_SERVO_ANGLE_LIMIT_WRITE    = 20,
        LEWAN_SERVO_ANGLE_LIMIT_READ     = 21,
        LEWAN_SERVO_VIN_LIMIT_WRITE      = 22,
        LEWAN_SERVO_VIN_LIMIT_READ       = 23,
        LEWAN_SERVO_TEMP_MAX_LIMIT_WRITE = 24,
        LEWAN_SERVO_TEMP_MAX_LIMIT_READ  = 25,
        LEWAN_SERVO_TEMP_READ            = 26,
        LEWAN_SERVO_VIN_READ             = 27,
        LEWAN_SERVO_POS_READ             = 28,
        LEWAN_SERVO_OR_MOTOR_MODE_WRITE  = 29,
        LEWAN_SERVO_OR_MOTOR_MODE_READ   = 30,
        LEWAN_SERVO_LOAD_OR_UNLOAD_WRITE = 31,
        LEWAN_SERVO_LOAD_OR_UNLOAD_READ  = 32,
        LEWAN_SERVO_LED_CTRL_WRITE       = 33,
        LEWAN_SERVO_LED_CTRL_READ        = 34,
        LEWAN_SERVO_LED_ERROR_WRITE      = 35,
        LEWAN_SERVO_LED_ERROR_READ       = 36
};

template <>
constexpr endianess_enum serialization_endianess< lewan_angle > = endianess_enum::BIG;
template <>
constexpr endianess_enum serialization_endianess< lewan_full_angle > = endianess_enum::BIG;
template <>
constexpr endianess_enum serialization_endianess< lewan_time > = endianess_enum::BIG;
template <>
constexpr endianess_enum serialization_endianess< lewan_voltage > = endianess_enum::BIG;
template <>
constexpr endianess_enum serialization_endianess< lewan_angular_velocity > = endianess_enum::BIG;

struct master_lewan_group
  : command_group<  //
        command< LEWAN_SERVO_MOVE_TIME_WRITE >::with_args< lewan_angle, lewan_time >,
        command< LEWAN_SERVO_MOVE_TIME_READ >,
        command< LEWAN_SERVO_MOVE_TIME_WAIT_WRITE >::with_args< lewan_angle, lewan_time >,
        command< LEWAN_SERVO_MOVE_TIME_WAIT_READ >,
        command< LEWAN_SERVO_MOVE_START >,
        command< LEWAN_SERVO_MOVE_STOP >,
        command< LEWAN_SERVO_ID_WRITE >::with_args< lewan_servo_id >,
        command< LEWAN_SERVO_ID_READ >,
        command< LEWAN_SERVO_ANGLE_OFFSET_ADJUST >::with_args< lewan_angle_offset >,
        command< LEWAN_SERVO_ANGLE_OFFSET_WRITE >,
        command< LEWAN_SERVO_ANGLE_OFFSET_READ >,
        command< LEWAN_SERVO_ANGLE_LIMIT_WRITE >::with_args< lewan_angle, lewan_angle >,
        command< LEWAN_SERVO_ANGLE_LIMIT_READ >,
        command< LEWAN_SERVO_VIN_LIMIT_WRITE >::with_args< lewan_voltage, lewan_voltage >,
        command< LEWAN_SERVO_VIN_LIMIT_READ >,
        command< LEWAN_SERVO_TEMP_MAX_LIMIT_WRITE >::with_args< lewan_temp >,
        command< LEWAN_SERVO_TEMP_MAX_LIMIT_READ >,
        command< LEWAN_SERVO_TEMP_READ >,
        command< LEWAN_SERVO_VIN_READ >,
        command< LEWAN_SERVO_POS_READ >,
        command< LEWAN_SERVO_OR_MOTOR_MODE_WRITE >::
            with_args< lewan_servo_mode, uint8_t, lewan_angular_velocity >,
        command< LEWAN_SERVO_OR_MOTOR_MODE_READ >,
        command< LEWAN_SERVO_LOAD_OR_UNLOAD_WRITE >::with_args< lewan_load >,
        command< LEWAN_SERVO_LOAD_OR_UNLOAD_READ >,
        command< LEWAN_SERVO_LED_CTRL_WRITE >::with_args< lewan_led_mode >,
        command< LEWAN_SERVO_LED_CTRL_READ >,
        command< LEWAN_SERVO_LED_ERROR_WRITE >::with_args< lewan_led_error >,
        command< LEWAN_SERVO_LED_ERROR_READ > >
{
};

using master_lewan_submessage = master_lewan_group::message_t;
using master_lewan_variant    = master_lewan_group::command_variant;

struct lewan_master_group
  : command_group<  //
        command< LEWAN_SERVO_MOVE_TIME_READ >::with_args< lewan_angle, lewan_time >,
        command< LEWAN_SERVO_MOVE_TIME_WAIT_READ >::with_args< lewan_angle, lewan_time >,
        command< LEWAN_SERVO_ID_READ >::with_args< lewan_servo_id >,
        command< LEWAN_SERVO_ANGLE_OFFSET_READ >::with_args< lewan_angle_offset >,
        command< LEWAN_SERVO_ANGLE_LIMIT_READ >::with_args< lewan_angle, lewan_angle >,
        command< LEWAN_SERVO_VIN_LIMIT_READ >::with_args< lewan_voltage, lewan_voltage >,
        command< LEWAN_SERVO_TEMP_MAX_LIMIT_READ >::with_args< lewan_temp >,
        command< LEWAN_SERVO_TEMP_READ >::with_args< lewan_temp >,
        command< LEWAN_SERVO_VIN_READ >::with_args< lewan_voltage >,
        command< LEWAN_SERVO_POS_READ >::with_args< lewan_full_angle >,
        command< LEWAN_SERVO_OR_MOTOR_MODE_READ >::
            with_args< lewan_servo_mode, uint8_t, lewan_angular_velocity >,
        command< LEWAN_SERVO_LOAD_OR_UNLOAD_READ >::with_args< lewan_load >,
        command< LEWAN_SERVO_LED_CTRL_READ >::with_args< lewan_led_mode >,
        command< LEWAN_SERVO_LED_ERROR_READ >::with_args< lewan_led_error > >
{
};

using lewan_master_submessage = lewan_master_group::message_t;
using lewan_master_variant    = lewan_master_group::command_variant;

template < typename Message >
class lewan_sequencer
{
        // soo, looking for 2x lewan_header_start , anything, lewan_packet_size

        enum states
        {
                HEADER_START_1,
                HEADER_START_2,
                SERVO_ID,
                PACKET_SIZE,
                PACKET_DATA
        };

        em::static_circular_buffer< uint8_t, 128 > buffer_;

        states      state_       = HEADER_START_1;
        std::size_t start_       = 0;
        std::size_t msg_size_    = 0;
        std::size_t ideal_batch_ = 2;

      public:
        template < typename Data, typename MsgCallback >
        bool load( const Data& data, MsgCallback&& cb )
        {
                auto data_view = em::view{ data };

                if ( data_view.empty() ) {
                        ideal_batch_ = 2;
                        return false;
                }
                buffer_.push_back( data_view.front() );
                bool has_fired = false;
                switch ( state_ ) {
                        case HEADER_START_1:
                                if ( data_view.front() == *lewan_header_start_v ) {
                                        state_ = HEADER_START_2;
                                } else {
                                        buffer_.clear();
                                }
                                break;
                        case HEADER_START_2:
                                if ( data_view.front() == *lewan_header_start_v ) {
                                        state_ = SERVO_ID;
                                } else {
                                        state_ = HEADER_START_1;
                                        buffer_.clear();
                                }
                                break;
                        case SERVO_ID:
                                state_ = PACKET_SIZE;
                                break;
                        case PACKET_SIZE:
                                msg_size_ = static_cast< std::size_t >( data_view.front() - 2 );
                                state_    = PACKET_DATA;
                                break;
                        case PACKET_DATA:
                                if ( msg_size_ != 0 ) {
                                        msg_size_ -= 1;
                                        break;
                                }
                                auto opt_msg = Message::make( buffer_ );
                                if ( opt_msg ) {
                                        has_fired = true;
                                        cb( *opt_msg );
                                } else {
                                        // ERROR
                                }

                                buffer_.clear();
                                state_ = HEADER_START_1;
                                break;
                }
                return load( em::tail( data_view ), cb ) || has_fired;
        }
};

using lewan_prefix_prot = protocol_tuple<>::
    with_items< lewan_header_start, lewan_header_start, lewan_servo_id, lewan_packet_size >;
using lewan_sufix_prot = protocol_tuple<>::with_items< lewan_checksum >;

template < typename Submessage >
struct lewan_protocol
{
        static constexpr std::size_t submsg_size = Submessage::max_size;
        static constexpr std::size_t size_lower_bound =
            lewan_prefix_prot::size + lewan_sufix_prot::size;

        using lewan_prefix_message = typename lewan_prefix_prot::message_t;
        using lewan_sufix_message  = typename lewan_sufix_prot::message_t;
        using message_t =
            prot_message< submsg_size + lewan_prefix_prot::size + lewan_sufix_prot::size,
                          lewan_protocol >;

        // TODO is stringify necessary?
        // TODO: make these smarter

        static std::string stringify( const message_t& msg )
        {
                std::stringstream ss;
                if ( msg.size() < size_lower_bound ) {
                        ss << "Error, lewan protocol message too small: ";
                        ss << em::view{ msg };
                        return ss.str();
                }

                std::size_t subsize = get_submsg_size( msg );

                ss << "["
                   << *lewan_prefix_message::make(
                          em::view_n( msg.begin(), lewan_prefix_prot::size ) )
                   << *Submessage::make(
                          em::view_n( msg.begin() + lewan_prefix_prot::size, subsize ) )
                   << *lewan_sufix_message::make(
                          em::view( msg.end() - lewan_sufix_prot::size, msg.end() ) )
                   << "]";
                return ss.str();
        };

        // TODO: check checksum
        static message_t create( lewan_servo_id id, Submessage subm )
        {
                std::array< uint8_t, message_t::max_size > buffer = { 0 };

                lewan_prefix_prot::serialize_at( make_static_view( buffer ),
                                                 lewan_header_start_v,
                                                 lewan_header_start_v,
                                                 id,
                                                 lewan_packet_size{ 0 } );

                auto end =
                    std::copy( subm.begin(), subm.end(), buffer.begin() + lewan_prefix_prot::size );

                end += lewan_sufix_prot::size;

                message_t msg{ buffer, static_cast< std::size_t >( end - buffer.begin() ) };

                // the size of the message is calculated based on reading the doc, so yeas -3 is
                // correct constat
                msg[3] = static_cast< uint8_t >( msg.size() - 3 );

                auto checksum =
                    em::sum( em::view{ msg.begin() + 2, msg.end() - 1 }, []( uint8_t val ) {  //
                            return uint16_t( val );
                    } );

                msg[msg.size() - 1] = static_cast< uint8_t >( ~( checksum & 0xff ) );

                return msg;
        }

        template < typename Message, typename... Visitors >
        static void match( Message&& m, Visitors&&... vis )
        {
                match( m, schpin::match( vis... ) );
        }

        template < typename Message, typename Visitor >
        static void match( Message&& m, Visitor&& vis )
        {
                lewan_servo_id id = lewan_servo_id::get< 0 >();
                lewan_prefix_prot::match(  //
                    em::view_n( m.begin(), lewan_prefix_prot::size ),
                    [&]( error_tag, protocol_error_record rec ) { vis( error_tag{}, rec ); },
                    [&]( lewan_header_start,
                         lewan_header_start,
                         lewan_servo_id i,
                         lewan_packet_size ) { id = i; } );
                std::size_t subsize = get_submsg_size( m );

                if ( lewan_prefix_prot::size + subsize > m.size() ) {
                        vis( error_tag{}, protocol_error_record{ INVALID_SIZE_E, 0 } );
                        return;
                }

                auto from = m.begin() + lewan_prefix_prot::size;
                auto to   = from + subsize;

                auto opt_msg = Submessage::make( em::view( from, to ) );
                if ( !opt_msg ) {
                        vis( error_tag{}, protocol_error_record{ INVALID_SIZE_E, 0 } );
                        return;
                }
                vis( id, *opt_msg );
        }

        template < typename Message >
        static std::size_t get_submsg_size( const Message& m )
        {
                return static_cast< std::size_t >( m[3] - 2 );
        }

        template < typename Message >
        static em::either< std::tuple< lewan_servo_id, Submessage >, protocol_error_record >
        extract( Message&& m )
        {
                em::either< std::tuple< lewan_servo_id, Submessage >, protocol_error_record > res =
                    protocol_error_record{};
                match(
                    m,
                    [&]( lewan_servo_id id, em::view< const uint8_t* > view ) {
                            auto opt_msg = Submessage::make( view );
                            if ( opt_msg ) {
                                    res = { std::make_tuple( id, *opt_msg ) };
                            } else {
                                    res = { protocol_error_record{ INVALID_SIZE_E, 0 } };
                            }
                    },
                    [&]( error_tag, protocol_error_record rec ) { res = rec; } );
                return res;
        }
};

using lewan_master_protocol = lewan_protocol< lewan_master_submessage >;
using lewan_master_message  = lewan_master_protocol::message_t;

using master_lewan_protocol = lewan_protocol< master_lewan_submessage >;
using master_lewan_message  = master_lewan_protocol::message_t;

// TODO: we need to define relationship query->response between messages
}  // namespace schpin
