#include "node/context.h"
#include "node/robot/view.h"
#include "node/state_machines/base.h"

#include <experimental/propagate_const>
#include <schpin_lib/lua/base.h>

#pragma once

namespace schpin
{

class lua_vm_interactive
{
        struct impl;

        std::experimental::propagate_const< std::unique_ptr< impl > > impl_ptr_;

      public:
        lua_vm_interactive();

        std::string get_json_def() const;

        em::either< std::optional< nlohmann::json >, slua_error >
        exec_bind( std::string, const nlohmann::json&, const node_context&, sm_feedback_f );

        opt_error< slua_error > exec( const std::string&, const node_context&, sm_feedback_f );

        ~lua_vm_interactive();
};

class lua_vm_machine
{
        struct impl;

        std::experimental::propagate_const< std::unique_ptr< impl > > impl_ptr_;

      public:
        lua_vm_machine( const std::string& );
        opt_error< slua_error > load( const std::string& code );
        opt_error< slua_error > on_cmd( fw_node_group_variant,
                                        const node_context&,
                                        sm_feedback_f );
        opt_error< slua_error > tick( const node_context& context, sm_feedback_f send_cb );
        bool                    finished();
        ~lua_vm_machine();
};

}  // namespace schpin
