#include "node/events.h"

#include <schpin_lib/error.h>

#pragma once

namespace schpin
{

using machine_feedback_var = std::variant< node_fw_group_message, set_body_pose_event >;

using sm_feedback_f       = std::function< ctl_id( machine_feedback_var ) >;
using state_machine_error = error< struct state_machine_error_tag >;

template < typename... Machines >
auto make_machine( const nlohmann::json& j ) -> std::variant< Machines... >
{
        std::variant< Machines... > res;
        // TODO: make for_each_Variant
        em::for_each( std::tuple< Machines... >{}, [&]( auto machine ) {
                using machine_type = decltype( machine );
                if ( j["machine"] == machine_type::json_keyword ) {
                        res = j["data"].get< machine_type >();
                }
        } );

        return res;
}

template < typename... Machines >
auto make_machine( std::istream& input ) -> std::variant< Machines... >
{
        return safe_load_json(
            input, [&]( const nlohmann::json& j ) { return make_machine< Machines... >( j ); } );
}

}  // namespace schpin
