
#include "blob/reactor.h"
#include "stm32g4xx_hal.h"

using namespace schpin;

using reactor = blob_reactor< 20 >;

UART_HandleTypeDef master_uart;
UART_HandleTypeDef lewan_uart;

reactor rec;

clk_time get_systime()
{
        return clk_time{ HAL_GetTick() };
}

struct stm32_commands
{
        static s_ctl_id servo_counter;

        void operator()( control_tag, const blob_master_group_message& group_msg )
        {
                auto m = blob_master_protocol::serialize( ctl_id{ 0 }, group_msg, checksum{} );
                HAL_UART_Transmit( &master_uart,
                                   const_cast< uint8_t* >( m.begin() ),
                                   static_cast< uint16_t >( m.size() ),
                                   200 );
        }
        s_ctl_id operator()( servo_tag, reply r, const master_lewan_message& m )
        {
                servo_counter++;
                HAL_UART_Transmit( &lewan_uart,
                                   const_cast< uint8_t* >( m.begin() ),
                                   static_cast< uint16_t >( m.size() ),
                                   200 );
                if ( r == reply::NO ) {
                        return servo_counter;
                }

                std::array< uint8_t, 10 > data;

                HAL_UART_Receive( &master_uart,
                                  const_cast< uint8_t* >( data.begin() ),
                                  static_cast< uint16_t >( data.size() ),
                                  200 );

                rec.on_servo_receive( get_systime(), servo_counter, data, stm32_commands{} );
                return servo_counter;
        }
};

s_ctl_id stm32_commands::servo_counter = 0;

int main()
{

        while ( 1 ) {
                rec.tick( get_systime(), stm32_commands{} );
        }
}
