#include "schpin_lib/chain.h"
#include "schpin_lib_exp/rviz.h"

#pragma once

namespace schpin
{

// Creates a basic marker for visuaization of links
inline visualization_msgs::Marker create_chain_link_marker( unsigned           id,
                                                            const std::string& frame,
                                                            const std::string& ns )
{
        visualization_msgs::Marker res = create_marker( ns + "-link", id, frame );

        res.type    = visualization_msgs::Marker::LINE_LIST;
        res.scale.x = 0.01;
        res.scale.y = 0.01;
        res.scale.z = 0.01;

        res.color.r = 0.f;
        res.color.g = 1.f;
        res.color.b = 0.f;

        return res;
}
// Creates a basic marker for visualization of pins
inline visualization_msgs::Marker create_chain_pin_marker( unsigned           id,
                                                           const std::string& frame,
                                                           const std::string& ns )
{
        visualization_msgs::Marker res = create_marker( ns + "-pin", id, frame );

        res.type    = visualization_msgs::Marker::SPHERE_LIST;
        res.scale.x = 0.01;
        res.scale.y = 0.01;
        res.scale.z = 0.01;

        res.color.r = 0.f;
        res.color.g = 1.f;
        res.color.b = 0.f;

        return res;
}

// Creates an array of visualizations as a line_list (for links) and sphere_list ( or pins). This
// needs a pin_to_point conversion function that returns a point in 3D for each pin of the chain.
template < typename Chain, typename PinToPointFunction >
visualization_msgs::MarkerArray create_chain_visualization( const Chain&         ch,
                                                            const std::string&   frame,
                                                            const std::string&   ns,
                                                            PinToPointFunction&& pin_to_point )
{
        visualization_msgs::Marker link_msg = create_chain_link_marker( 0, frame, ns );
        visualization_msgs::Marker pins_msg = create_chain_pin_marker( 1, frame, ns );

        auto f = [&]( const auto& pin ) -> geometry_msgs::Point {
                return point_to_msg( pin_to_point( pin ) );
        };

        if ( ch.begin() != ch.end() ) {
                pins_msg.points.push_back( f( ch.begin()->source_pin ) );
        }
        for ( const auto& view : ch ) {
                link_msg.points.push_back( f( view.source_pin ) );
                link_msg.points.push_back( f( view.target_pin ) );
                pins_msg.points.push_back( f( view.target_pin ) );
        }

        visualization_msgs::MarkerArray res;
        res.markers = { link_msg, pins_msg };
        return res;
}

}  // namespace schpin
