#include <functional>
#include <nlohmann/json.hpp>

#pragma once

namespace schpin
{

// Color in range of 0-1.f
struct color
{
        float r = 0.f;
        float g = 0.f;
        float b = 0.f;
};

constexpr bool operator!=( const color& lh, const color& rh )
{
        return lh.r != rh.r && lh.g != rh.g && lh.b != rh.b;
}

inline void to_json( nlohmann::json& j, const color& c )
{
        j["r"] = c.r;
        j["g"] = c.g;
        j["b"] = c.b;
}

inline void from_json( const nlohmann::json& j, color& c )
{
        c.r = j.at( "r" ).get< float >();
        c.g = j.at( "g" ).get< float >();
        c.b = j.at( "b" ).get< float >();
}

}  // namespace schpin

template <>
struct std::hash< schpin::color >
{
        std::size_t operator()( const schpin::color& c )
        {
                std::hash< float > f{};
                return f( c.r ) ^ f( c.g ) ^ f( c.b );
        }
};
