#include "schpin_lib/xml.h"
#include "schpin_lib_exp/algorithm.h"

#pragma once

namespace schpin
{

struct srdf_information
{
        std::vector< std::string > root_joints;
        std::string                body_link;
};

inline em::either< srdf_information, xml_storage_error > load_srdf_info( const std::string& srdf )
{
        std::stringstream srdf_istream{ srdf };
        return safe_load_xml< srdf_information >(
            srdf_istream,
            [&]( const pt::ptree& tree ) -> em::either< srdf_information, xml_storage_error > {
                    auto leg_group_iter = em::find_if(
                        tree.get_child( "robot" ),
                        em::uncurry | [&]( const std::string&, const pt::ptree& child ) {
                                return child.get< std::string >( "<xmlattr>.name", "" ) == "legs";
                        } );

                    if ( leg_group_iter == tree.get_child( "robot" ).end() ) {
                            return { E( xml_storage_error ) << "SRDF XML file does not have group "
                                                               "with name 'legs'" };
                    }

                    std::vector< std::string > root_joints = filter_optionals(
                        leg_group_iter->second,
                        em::uncurry | [&]( const std::string& key,
                                         const pt::ptree& child ) -> std::optional< std::string > {
                                if ( key != "leg" ) {
                                        return {};
                                }
                                return { child.get< std::string >( "<xmlattr>.root_joint" ) };
                        } );

                    return { srdf_information{ root_joints, "body" } };
            } );
}

}  // namespace schpin
