#include "schpin_lib/error.h"

#include <boost/property_tree/ptree.hpp>
#include <boost/property_tree/xml_parser.hpp>
#include <filesystem>

#pragma once

namespace schpin
{

namespace pt = ::boost::property_tree;

using xml_storage_error = error< struct xml_storage_error_tag >;

template < typename T, typename UnaryFunction >
em::either< T, xml_storage_error > safe_load_xml( std::istream& sstream, UnaryFunction f )
{
        try {
                pt::ptree tree;
                pt::read_xml( sstream, tree );
                return {f( tree )};
        }
        catch ( const pt::xml_parser::xml_parser_error& e ) {
                return {E( xml_storage_error ) << "Failed to parse the XML file:" << e.what()};
        }
        catch ( const pt::ptree_error& e ) {
                return {E( xml_storage_error ) << "Failed to extract XML data:" << e.what()};
        }
}

}  // namespace schpin
