#include <optional>
#include <ostream>

#pragma once

namespace schpin
{

template < typename T, T MinVal, T MaxVal >
class bounded
{
        T val_;

        constexpr bounded( T val )
          : val_( val )
        {
        }

      public:
        static constexpr T min_val = MinVal;
        static constexpr T max_val = MaxVal;

        template <
            typename U,
            U OtherMin,
            U OtherMax,
            std::enable_if_t< std::is_integral_v< U > && std::is_integral_v< T > >* = nullptr,
            std::enable_if_t< ( min_val <= OtherMin && OtherMax <= max_val ) >*     = nullptr >
        constexpr bounded( bounded< U, OtherMin, OtherMax > other )
          : val_( static_cast< T >( *other ) )
        {
        }

        template < T Val >
        static constexpr bounded get()
        {
                static_assert( min_val <= Val && Val <= max_val );
                return bounded{ Val };
        }

        static std::optional< bounded< T, min_val, max_val > > make( T val )
        {
                if ( val < min_val ) {
                        return {};
                }
                if ( val > max_val ) {
                        return {};
                }
                return bounded{ val };
        }

        static bounded< T, min_val, max_val > min()
        {
                return bounded{ min_val };
        }

        static bounded< T, min_val, max_val > max()
        {
                return bounded{ max_val };
        }

        constexpr bounded()
          : val_( min_val )
        {
        }

        T operator*() const
        {
                return val_;
        }

        explicit operator T() const
        {
                return val_;
        }
};

template < typename T, T MinVal, T MaxVal >
std::ostream& operator<<( std::ostream& os, const bounded< T, MinVal, MaxVal >& b )
{
        if constexpr ( std::is_same_v< T, uint8_t > ) {
                return os << int( *b );
        } else {
                return os << *b;
        }
}

// TODO: there should be crtp base class for the overloads
template < typename T, T MinVal, T MaxVal >
inline bool operator==( const bounded< T, MinVal, MaxVal >& lh,
                        const bounded< T, MinVal, MaxVal >& rh )
{
        return *lh == *rh;
}

template < typename T, T MinVal, T MaxVal >
inline bool operator!=( const bounded< T, MinVal, MaxVal >& lh,
                        const bounded< T, MinVal, MaxVal >& rh )
{
        return !( lh == rh );
}

template < typename T, T MinVal, T MaxVal >
inline bool operator<=( const bounded< T, MinVal, MaxVal >& lh,
                        const bounded< T, MinVal, MaxVal >& rh )
{
        return *lh <= *rh;
}
template < typename T, T MinVal, T MaxVal >
inline bool operator>=( const bounded< T, MinVal, MaxVal >& lh,
                        const bounded< T, MinVal, MaxVal >& rh )
{
        return *lh >= *rh;
}
template < typename T, T MinVal, T MaxVal >
inline bool operator<( const bounded< T, MinVal, MaxVal >& lh,
                       const bounded< T, MinVal, MaxVal >& rh )
{
        return *lh < *rh;
}
template < typename T, T MinVal, T MaxVal >
inline bool operator>( const bounded< T, MinVal, MaxVal >& lh,
                       const bounded< T, MinVal, MaxVal >& rh )
{
        return *lh > *rh;
}

// TODO check if we need this
struct bounded_out_of_range_error
{
};

}  // namespace schpin
