#include <schpin_lib/algorithm.h>

#pragma once

namespace schpin
{

template < typename Function, typename... Tags >
struct visitor_callback
{
        using tag_tuple = std::tuple< Tags... >;

        Function f;
};

template < typename... Callbacks >
class visitor_driver
{
        using callbacks_tuple = std::tuple< Callbacks... >;

        callbacks_tuple callbacks_;

      public:
        visitor_driver( Callbacks... cbs )
          : callbacks_( std::move( cbs )... )
        {
        }

        template < typename Tag, typename... Args >
        decltype( auto ) call( Args&&... args )
        {
                em::for_each( callbacks_, [&]( auto& cb ) {
                        using callback = std::remove_reference_t< decltype( cb ) >;
                        if constexpr ( em::tuple_has_type_v< Tag, typename callback::tag_tuple > ) {
                                cb.f( std::forward< Args >( args )... );
                        }
                } );
        }
};

}  // namespace schpin
