#include "schpin_lib_exp/algorithm.h"
#include "schpin_lib_exp/console.h"

namespace schpin
{

using namespace type_console;

const completion_context* COMPLETION_CANDIDATES = nullptr;

char** completer( const char* text, int /*start*/, int /*end*/ )
{
        // Don't do filename completion even if our generator finds no matches.
        rl_attempted_completion_over = 1;

        std::vector< std::string_view > words = split( text );

        // find all perfect matches
        const completion_context* node = COMPLETION_CANDIDATES;
        completion_candidates     cc   = find_candidates( em::view( words ), node );

        if ( cc.candidates.empty() ) {
                return NULL;
        }

        if ( cc.candidates.size() == 1 ) {
                char** res = reinterpret_cast< char** >( malloc( sizeof( char* ) * 2 ) );
                res[0]     = strdup( ( cc.prefix + std::string{ cc.candidates.front() } ).c_str() );
                res[1]     = NULL;
                return res;
        }

        // get prefix
        std::size_t len = em::max_elem( cc.candidates, [&]( const std::string_view& view ) {  //
                return view.size();
        } );
        std::size_t i   = *em::find_if( em::range( len ), [&]( std::size_t i ) {
                return em::any_of( em::tail( cc.candidates ),
                                   [&]( const std::string_view& view ) {  //
                                           return view[i] != cc.candidates.front()[i];
                                   } );
        } );
        // build result array

        std::size_t res_size = cc.candidates.size() + 2;
        char**      res      = reinterpret_cast< char** >( malloc( sizeof( char* ) * res_size ) );
        res[0] = strdup( ( cc.prefix + std::string{ cc.candidates[0].substr( 0, i ) } ).c_str() );
        for ( std::size_t i : em::range( cc.candidates.size() ) ) {
                res[i + 1] = strdup( ( cc.prefix + std::string{ cc.candidates[i] } ).c_str() );
        }
        res[res_size - 1] = NULL;
        return res;
}

std::vector< std::string > type_console::prompt( const std::vector< std::string >& p,
                                                 const completion_context*         root )
{
        for ( const std::string& item : p ) {
                root = *em::find_if( root->children, [&]( const completion_context* child ) {  //
                        return child->key == item;
                } );
        }
        COMPLETION_CANDIDATES = root;

        std::stringstream ss;
        ss << color::MAGENTA << em::joined( p, std::string{ "  " } ) << color::RESET << " > ";

        rl_attempted_completion_function   = completer;
        static char word_break_chars       = '\0';
        rl_completer_word_break_characters = &word_break_chars;
        std::cout << std::unitbuf;

        char* buf = readline( ss.str().c_str() );

        if ( buf == nullptr ) {
                return {};
        }

        if ( strlen( buf ) == 0 ) {
                free( buf );
                return {};
        }

        add_history( buf );

        std::vector< std::string > res = map_f_to_v(
            split( buf ), [&]( std::string_view view ) { return std::string{ view }; } );
        free( buf );
        return res;
}

}  // namespace schpin
