#include <gtest/gtest.h>
#include <schpin_lib_exp/console.h>

struct compl_test_case
{
        std::string                input;
        std::string                prefix;
        std::vector< std::string > candidates;
};

using namespace schpin;
using namespace schpin::type_console;

TEST( Console, findCandidates )
{
        // foo -> foo -> boo
        //            -> bo
        //     -> fo  -> bo
        //     -> ofo
        completion_context context1{ "boo", {} };
        completion_context context2{ "bo", {} };
        completion_context context3{ "foo", { &context1, &context2 } };
        completion_context context4{ "bo", {} };
        completion_context context5{ "fo", { &context4 } };
        completion_context context6{ "ofo", {} };
        completion_context context7{ "foo", { &context5, &context6, &context3 } };
        completion_context root{ "", { &context7 } };

        std::vector< compl_test_case > tests = { { "foo foo b", "foo foo ", { "boo", "bo" } },
                                                 { "foo f", "foo ", { "fo", "foo" } },
                                                 { "foo ofo", "foo ofo ", {} },
                                                 { "", "", { "foo" } },
                                                 { "boo", "", {} } };

        for ( const compl_test_case& test : tests ) {
                std::vector< std::string_view > words = split( test.input );
                completion_candidates           cc    = find_candidates( em::view{ words }, &root );

                EXPECT_EQ( cc.prefix, test.prefix ) << em::view{ words };
                EXPECT_TRUE( em::equal( cc.candidates, test.candidates ) )
                    << em::view{ cc.candidates } << ", " << em::view{ test.candidates };
        }
}
