#include "schpin_lib/geom/line.h"
#include "schpin_test/point_test_util.h"

#include <gtest/gtest.h>

TYPED_TEST( PointTest, line_point_distance )
{
        static constexpr unsigned n = TypeParam::value;
        line< n, world_frame >    test_line{ this->zero_p_, this->unit_p_ };

        ASSERT_NEAR( *distance_of( test_line, this->unit_p_ ), *em::distance( 0. ), 1e-6 );
        ASSERT_NEAR( *distance_of( test_line, this->zero_p_ ), *em::distance( 0. ), 1e-6 );
        ASSERT_NEAR( *distance_of( test_line, -this->unit_p_ ), *length_of( this->unit_p_ ), 1e-6 );
}

TYPED_TEST( PointTest, perpendicular )
{
        static constexpr unsigned n = TypeParam::value;
        point< 2, world_frame >   p{ { n, 0 } };
        point< 2, world_frame >   zero_p{ { 0, 0 } };
        point< 2, world_frame >   unit_p{ { n, n } };

        em::distance diag_dist{ float( sqrt( 2 * n * n ) / 2 ) };

        line< 2, world_frame > test_line{ zero_p, unit_p };
        ASSERT_NEAR( *distance_of( test_line, p ), *diag_dist, 1e-6 );

        line< 2, world_frame > reversed_line{ unit_p, zero_p };
        ASSERT_NEAR( *distance_of( reversed_line, p ), *diag_dist, 1e-6 );

        line< 2, world_frame > diag_line{ point< 2, world_frame >{ { 0, n } },
                                          point< 2, world_frame >{ { n, 0 } } };
        ASSERT_NEAR( *distance_of( diag_line, zero_p ), *diag_dist, 1e-6 );
        ASSERT_NEAR( *distance_of( diag_line, unit_p ), *diag_dist, 1e-6 );
}
