#include "collission_interfaces.h"
#include "schpin_lib/geom/point.h"

#pragma once

namespace schpin
{

// Query for the collision detection system for point A
template < std::size_t N, typename TagT >
class point_collision_query : i_collision_item< const point< N, TagT >*, const vec< N >* >
{
      public:
        using tag = TagT;

      private:
        std::array< point< N, tag >, 1 > point_;

      public:
        explicit point_collision_query( const point< N, tag >& a )
          : point_( {a} )
        {
        }

        using item_type                = point< N, tag >;
        using point_iterator           = const point< N, tag >*;
        using separation_axis_iterator = const vec< N >*;

        em::view< point_iterator > get_points() const final
        {
                return em::view{point_};
        }

        em::view< separation_axis_iterator > get_separation_axes() const final
        {
                return {nullptr, nullptr};
        }

        em::view< separation_axis_iterator > get_intersection_axes() const final
        {
                return {nullptr, nullptr};
        }
};

// Query for the collision detection system for point A
template < std::size_t N, typename TagT >
class fast_point_collision_query : i_fast_collision_item< point_collision_query< N, TagT >, N >
{
      public:
        using tag       = TagT;
        using full_type = point_collision_query< N, TagT >;

      private:
        point< N, tag > point_;

      public:
        explicit fast_point_collision_query( const point< N, tag >& a )
          : point_( a )
        {
        }

        point< N, tag > get_min_sphere_position() const final
        {
                return point_;
        }

        em::radius get_min_sphere_radius() const final
        {
                return em::radius{0.f};
        }
        point_collision_query< N, tag > get_full() const final
        {
                return point_collision_query{point_};
        }

        bool is_leaf() const
        {
                return true;
        }

        std::vector< fast_point_collision_query > get_children() const
        {
                return {};
        }
};
}  // namespace schpin
