#include "base/physics.h"
#include "schpin_lib/geom/point.h"
#include "schpin_lib/view.h"
#include "util.h"

#pragma once

namespace schpin
{

enum class collision_mode
{
        FULL = 0,
        FAST = 1
};

template < typename PointIteratorT, typename SeparationAxisIterator >
class i_collision_item  // NOLINT(cppcoreguidelines-special-member-functions)
{
      public:
        using point_iterator           = PointIteratorT;
        using separation_axis_iterator = SeparationAxisIterator;

        virtual em::view< point_iterator >           get_points() const            = 0;
        virtual em::view< separation_axis_iterator > get_separation_axes() const   = 0;
        virtual em::view< separation_axis_iterator > get_intersection_axes() const = 0;
        virtual ~i_collision_item()                                                = default;
};

template < typename FullT, std::size_t Nu >
class i_fast_collision_item
{
      public:
        using full_type                = FullT;
        static constexpr std::size_t n = Nu;
        using tag                      = typename full_type::tag;

        virtual point< n, tag > get_min_sphere_position() const = 0;
        virtual em::radius      get_min_sphere_radius() const   = 0;
        virtual full_type       get_full() const                = 0;
        virtual ~i_fast_collision_item()                        = default;
};

}  // namespace schpin
