#include "robot.h"

#pragma once

namespace schpin
{

void print_robot_model( const robot& rob )
{
        for ( const leg< 3 >& pleg : rob.get_legs() ) {
                SCHPIN_INFO_LOG( "robot.print", "Printing model for leg " << pleg.get_id() );

                auto& root = pleg.get_model().get_root_joint();

                std::function< void( const joint& j, std::string ) > f =
                    [&]( const joint& j, const std::string& prefix ) {
                            SCHPIN_INFO_LOG( "robot.print",
                                             prefix << "| " << j.name << " - " << j.offset << " - "
                                                    << j.sub_link.name );
                            if ( std::holds_alternative< revolute_joint >( j.jclass ) ) {
                                    const revolute_joint& rj =
                                        std::get< revolute_joint >( j.jclass );
                                    SCHPIN_INFO_LOG( "robot.print",
                                                     prefix << "+ " << rj.axis << " - ("
                                                            << rj.min_angle << "-" << rj.max_angle
                                                            << ") - " << rj.state_space_steps
                                                            << " - " << rj.max_velocity );
                            } else {
                                    SCHPIN_INFO_LOG( "robot.print", prefix << "+ fixed" );
                            }
                            if ( j.sub_link.child_ptr ) {
                                    f( *j.sub_link.child_ptr, prefix + "  " );
                            }
                    };
                f( root, "" );
        }
}

}  // namespace schpin
