#include "robot/storage.h"

#include "leg/storage.h"

namespace schpin
{
em::either< robot, robot_error > load_robot( const std::filesystem::path& cache_prefix,
                                             const urdf::Model&           model,
                                             const srdf_information&      srdf_info )
{
        return assemble_left_collect_right(  //
                   load_legs( cache_prefix, model, srdf_info )
                       .convert_right( [&]( leg_boot_error&& e ) -> robot_error {
                               return E( robot_error ).attach( e ) << "Errors occured during the "
                                                                      "load of leg";
                       } ),
                   load_body( model, "body" )
                       .convert_right( [&]( body_boot_error&& e ) -> robot_error {
                               return E( robot_error ).attach( e ) << "Errors occured during the "
                                                                      "load of body";
                       } ) )
            .convert_left( em::uncurry |
                           [&]( leg_container< leg< 3 > > legs, body b ) {
                                   return robot( std::move( b ), std::move( legs ) );
                           } )
            .convert_right( [&]( auto&& errors ) -> robot_error {
                    return E( robot_error ).group_attach( errors ) << "Failed to load the robot";
            } );
}
}  // namespace schpin
