#include "robot.h"
#include "schpin_lib/srdf.h"

#pragma once

namespace schpin
{

em::either< robot, robot_error > load_robot( const std::filesystem::path& cache_prefix,
                                             const urdf::Model&           model,
                                             const srdf_information&      srdf_info );

inline em::either< robot, robot_error > load_robot_ros( const ros::NodeHandle& handle,
                                                        const reactor_config&  cfg )
{
        urdf::Model model;
        model.initParam( cfg.robot_description_topic );

        std::string srdf;
        handle.getParam( cfg.robot_semantics_topic, srdf );

        return load_srdf_info( srdf )
            .convert_right( [&]( const xml_storage_error& e ) {
                    return E( robot_error ).attach( e ) << "Failed to load robot model, when "
                                                           "loading the srdf "
                                                           "definition";
            } )
            .bind_left( [&]( const srdf_information& srdf_info ) {
                    return load_robot( cfg.cache_prefix, model, srdf_info );
            } );
}
}  // namespace schpin
